//
//  Path.cpp
//  Session 9
//
//  Created by Luis Angel on 5/28/15.
//  Copyright (c) 2015 Luis Angel. All rights reserved.
//

#include <iostream>
#include <cmath>

using namespace std;

class Point
{
public:
	double x;
	double y;
	
	/**
	 * @brief Default constructor.
	 */
	Point()
	{
		x = y = 0;		// Notice, using x and y without 'this'.
	}
	
	/**
	 * @brief Constructor with arguments.
	 */
	Point( double x, double y )
	{
		this->x = x;	// Notice, using 'this' with '->'.
		this->y = y;
	}
	
	/**
	 * @brief Distance function.
	 */
	double distance( const Point& p ) const
	{
		double diffX = this->x - p.x;
		double diffY = (*this).y - p.y;		// Notice 'this' with '*' and '.'.
		
		return sqrt( diffX*diffX + diffY*diffY );
	}
};

class Path
{
private:
	Point *points[100];		// An array of 100 pointer to Point objects.
	int pointCount;			// How many points we ha created.
	double length;			// Path's length.
	
public:
	/**
	 * @brief Constructor.
	 */
	Path()
	{
		pointCount = 0;		// No points.
		length = 0;
	}
	
	/**
	 * @brief Destructor.
	 */
	~Path()
	{
		// Free all points memory.
		for( int I = 0; I < pointCount; I++ )
			delete points[I];
		
		cout << "Destructor has been called!" << endl;
	}
	
	/**
	 * @brief Add a point.
	 */
	void add( double x, double y )
	{
		if( pointCount < 100 )
		{
			// Create a new point.
			points[pointCount] = new Point( x, y );
			
			// Increase length.
			if( pointCount > 0 )
				length += points[pointCount-1]->distance( *points[pointCount] );
			
			pointCount++;
		}
		else
			cout << "We can't add more points to the path!" << endl;
	}
	
	/**
	 * @brief Get a point.
	 */
	Point getPoint( int I ) const
	{
		I = max( 0, min( pointCount-1, I ) );	// I must be in [0, pointCount-1].
		return *points[I];
	}
	
	/**
	 * @brief Get total distance or length in this path.
	 */
	double getTotalDistance() const
	{
		return length;
	}
};

int main()
{
	Path p;
	p.add( 0, 0 );		// A square.
	p.add( 1, 0 );
	p.add( 1, 1 );
	p.add( 0, 1 );
	p.add( 0, 0 );
	
	cout << p.getTotalDistance() << endl;
	cout << "The last point was: (" << p.getPoint(4).x
		 << ", " << p.getPoint(4).y << ")" << endl;
	
	return 0;
}

